function [hBode,hFig]=bode_asymptotic(b,num,den,w_min,w_max,col)
%% Parameters
% b-> 0 (only asymptotic); 1 (both asymptotic and regular)
% num-> numerator of the transfer function
% den-> denominator of the transfer function
% w_min -> lower bound of the frequency for the diagram
% w_max -> upper bound of the frequency for the diagram
% col -> e.g 'r' -> color for the drawing of the asymptotic diagram
%
% %% Example of Usage
% % Parameters
% b = 1;
% num = [1 2];
% den = [1 11 28];
% w_min = 10^-2;
% w_max = 10^2;
% col = 'm';
%
% % Minimal Set of Parameters
% bode_asymptotic(b,num,den);
%
% % Full Set of Parameters
% bode_asymptotic(b,num,den,w_min,w_max,col);
%
% % Alternative Usage
% s = tf('s');
% P = (s+2)/(s^2 + 11*s + 28)
%
% % Minimal Set of Parameters
% bode_asymptotic(b,P);
%
% % Full Set of Parameters
% bode_asymptotic(b,P,w_min,w_max,col);



% Number of Arguments
ni = nargin;

switch (ni)
    % It must be a trasfer function
    case 2
        den = num.den{1};
        num= num.num{1};
        col = 'r';
    case 3
        % it must be a transfer function plus color
        if (ischar(den))
            col = den;
            den = num.den{1};
            num= num.num{1};
        else % It must be a transfer function with num and den
            col = 'r';
        end
        % It must be a transfer function plus frequency range
    case 4
        w_max = w_min;
        w_min = den;
        den = num.den{1};
        num= num.num{1};
        col = 'r';
    case 5
        % It must be a transfer function plus frequency range plus color
        if (ischar(w_max))
            col=w_max;
            w_max = w_min;
            w_min = den;
            den = num.den{1};
            num= num.num{1};
        else
            % It is a regular set of argument let's just select a default
            % color
            col = 'r';
        end
end

% % It must be a trasfer function
% if ni==2
%     den = num.den{1};
%     num= num.num{1};
%     col = 'r';
% end
%
% if ni==3
%     % it must be a transfer function plut color
%     if (ischar(den))
%         col = den;
%         den = num.den{1};
%         num= num.num{1};
%
%     end
% end
%
% % It must be a transfer function plus frequency range
% if ni==4
%     w_max = w_min;
%     w_min = den;
%     den = num.den{1};
%     num= num.num{1};
%     col = 'r';
% end
%
% if ni==5
%     % It must be a transfer function plus frequency range plus color
%     if (ischar(w_max))
%         col=w_max;
%         w_max = w_min;
%         w_min = den;
%         den = num.den{1};
%         num= num.num{1};
%     else
%         col = 'r';
%     end
% end



% Define Transfer Functions
s=tf('s');
G=tf(num,den);

% Detect Zeros and Poles
zroots=roots(num);
proots=roots(den);
% Find number of zeros at origin
[iz,~]=find(abs(zroots)==0);
numz=length(iz);
% Find number of poles at origin
[ip,~]=find(abs(proots)==0);
nump=length(ip);
pend=numz-nump;
h=nump-numz;

% Extrapolate zeros and poles different from zeros
sz=[1:length(zroots)];
sp=[1:length(proots)];
zroots=zroots(setdiff(sz,iz),:);
proots=proots(setdiff(sp,ip),:);


% Compact Form for Zeros and Poles to where the imaginary part is used to
% encode the zero/pole sign
zroots2=abs(zroots)-1i*real(zroots)./abs(zroots);
proots2=abs(proots)-1i*real(proots)./abs(proots);


% Compact Representation for Zeros and Poles
% Each row looks like [a b c]
% a=1 -> num; a=-1->den
zz=ones(size(zroots2,1),3);
zz(:,2)=real(zroots2);
zz(:,3)=imag(zroots2);
po=-ones(size(proots2,1),3);
po(:,2)=real(proots2);
po(:,3)=imag(proots2);


% First let's plot the exact bode diagram
if ni>3
    hBode=bodeplot(G,{w_min,w_max},'--');
else
    hBode=bodeplot(G,'--');
end
% Get current figure to access the axes
hFig = gcf();
axesObjs = get(hFig, 'Children');

dataObjs = get(axesObjs, 'Children');
% set(findobj(dataObjs,'Tag','Line2'),'LineStyle','none');
idx = zeros(2,1);
jj=1;
for i=1:numel(dataObjs)
    if (~isempty(dataObjs{i}(1).Children))
        %             disp(['is field ' num2str(i) ])
        if (b==0)
            dataObjs{i}(1).Children.LineStyle='none';
        end
        idx(jj)=i;
        jj=jj+1;
    end
end
%     dataObjs{2}(1).Children.LineStyle='none';
%     dataObjs{3}(1).Children.LineStyle='none';


% Get BodePlot Option
opt = getoptions(hBode);


%tracciamento moduli
pp_mag=po;
zz_mag=zz;
vect=[zz_mag;pp_mag];
vect=sortrows(vect,2);


interval=zeros(1,size(vect,1)+2);
interval(:,[2:(size(vect,1)+1)])=vect(:,2)';
try
    % The frequency range is decided by the bodeplot. Here I only need to
    % be sure that the asymptotic line is drawn till I need it, so I have
    % to check whether my limits are outside of the bodeplot or not, if
    % they are outside i need to extend the drawing area.
    interval(1,1)=min(0.01*vect(1,2),opt.XLim{1}(1));
    interval(1,end)=max(100*vect(end,2),opt.XLim{1}(2));
catch
    interval(1,1)=0.01*1;
    interval(1,2)=100*1;
end

g=(abs(freqresp(tf(num,den),interval(1))));
y(1)=20*log10(g);
indice=2;
y_old=y(1);
i_old=interval(1);
for i=interval(:,[2:length(interval)-1])
    y(indice)=20*pend*log10(i)+y_old-20*pend*log10(i_old);
    pend=pend+vect(indice-1,1);
    i_old=i;
    y_old=y(indice);
    indice=indice+1;
end
y(indice)=20*pend*log10(interval(end))+y_old-20*pend*log10(i_old);
yy = (abs(freqresp(tf(num,den),interval(end))));


% Second let's plot the magnitude of the asymptotic diagram
% axes(axesObjs(3))
axes(axesObjs(idx(2)))
hold on
semilogx(interval,y,col);
hold off;

% Asymptotic Phase Computation
clear interval;
[ii,jj]=find(vect(:,3)<0);
vect(ii,1)=-(vect(ii,1));
indice=1;
for i=1:size(vect,1)
    interval(1,indice)=0.1*vect(i,2);
    interval(2,indice)=vect(i,1);
    indice=indice+1;
    interval(1,indice)=vect(i,2);
    interval(2,indice)=0;
    indice=indice+1;
    interval(1,indice)=10*vect(i,2);
    interval(2,indice)=-vect(i,1);
    indice=indice+1;
end

try
    interval=interval';
    interval=sortrows(interval,1);
    interval=interval';
catch
    interval(:,1)=[0.01*1;0];
    interval(:,2)=[100*1;0];
    interval=interval';
    interval=sortrows(interval,1);
    interval=interval';
end


% Phase Offset Detection
interval=[[opt.XLim{1}(1);0] interval [opt.XLim{1}(2);0]];

if (dcgain(s^h*G)<0)
    % Pretend Gain to be positive
    po=rad2deg(angle(freqresp(s^h*-G,interval(1))));
    % Add contribution of negative gain
    po = po - 180;
    % Add contribution of pole at origin
    po = po - 90*h;
else
    % Gain is positive
    po=rad2deg(angle(freqresp(s^h*G,interval(1))));
    po = po - 90*h;
end


% Phase
x=round(po/90);
p=90*x;

yf(1)=p;
indice=2;
yf_old=yf(1);
i_old=interval(1,1);
pend=0;
for i=interval(1,[2:size(interval,2)-1])
    yf(indice)=45*pend*log10(i)+yf_old-45*pend*log10(i_old);
    pend=pend+interval(2,indice);
    i_old=i;
    yf_old=yf(indice);
    indice=indice+1;
end

yf(indice)=yf_old;
% subplot(2,1,2)



% Phase Correction for Bode Plot
opt.PhaseMatching = 'on';
opt.PhaseMatchingFreq = interval(1);
opt.PhaseMatchingValue = po;
setoptions(hBode,opt);
hold on;
% Third let's plot the phase of the asymptotic diagram
% axes(axesObjs(2))
axes(axesObjs(idx(1)))
hold on
semilogx(interval(1,:),yf,col);
grid on;
hold off;

% % Change Font Size
% set(findall(h,'-property','FontSize'),'FontSize',20)
% % Change Line Width
% set(findall(h,'Type','line'),'LineWidth',2)

